@@ -1,10 +1,10 @@
 # @package      hubzero-submit-common
 # @file         JobOutput.py
 # @author       Steven Clark <clarks@purdue.edu>
-# @copyright    Copyright (c) 2012 HUBzero Foundation, LLC.
+# @copyright    Copyright (c) 2012-2013 HUBzero Foundation, LLC.
 # @license      http://www.gnu.org/licenses/lgpl-3.0.html LGPLv3
 #
-# Copyright (c) 2012 HUBzero Foundation, LLC.
+# Copyright (c) 2012-2013 HUBzero Foundation, LLC.
 #
 # This file is part of: The HUBzero(R) Platform for Scientific Collaboration
 #
@@ -33,9 +33,33 @@ from xml.dom import minidom, Node
 from hubzero.submit.LogMessage    import logID as log
 from hubzero.submit.CommentedFile import CommentedFile
 
+def getDaxExecutables(daxPath):
+   daxExecutables = []
+   if os.path.exists(daxPath):
+      daxDoc = minidom.parse(daxPath)
+      rootNode = daxDoc.documentElement
+      children = rootNode.getElementsByTagName("executable")
+
+      for child in children:
+         installed = child.getAttribute('installed')
+         if not installed.lower() in ('true','t'):
+            for grandChild in child.childNodes:
+               if grandChild.nodeType == Node.ELEMENT_NODE:
+                  if grandChild.tagName == "pfn":
+                     url = grandChild.getAttribute("url")
+                     if url.startswith("file://"):
+                        executablePath = url[len("file://"):]
+                        daxExecutables.append(executablePath)
+   else:
+      log("dax %s file is missing" % (daxPath))
+
+   return(daxExecutables)
+
+
 def getJobOutputContent(parentNode,
                         elementTag,
-                        elementId,dataTag):
+                        elementId,
+                        dataTag):
    jobOutputContent = None
    content = []
    children = parentNode.getElementsByTagName(elementTag)
@@ -83,8 +107,13 @@ def getJobExecutable(parentNode):
 
 
 def processPegasusFiles(instanceDirectory,
+                        scratchDirectory,
                         timeResultsBase):
-   jobStateLogPath = os.path.join(instanceDirectory,'work','pegasus','jobstate.log')
+   pegasusFileDirectory = os.path.join(scratchDirectory,'work')
+   jobStateLogPath = os.path.join(pegasusFileDirectory,'jobstate.log')
+   if not os.path.exists(jobStateLogPath):
+      pegasusFileDirectory = os.path.join(instanceDirectory,'work')
+      jobStateLogPath = os.path.join(pegasusFileDirectory,'jobstate.log')
    if os.path.exists(jobStateLogPath):
       condorIds = {}
       fpJobStateLog = open(jobStateLogPath,'rb')
@@ -155,7 +184,7 @@ def processPegasusFiles(instanceDirectory,
 #                                                        row[columnRuntime]))
 
                         jobOutputFile = "%s.out.%s" % (row[columnJob],str(jobTry).zfill(3))
-                        jobOutputPath = os.path.join(instanceDirectory,'work','pegasus',jobOutputFile)
+                        jobOutputPath = os.path.join(pegasusFileDirectory,jobOutputFile)
                         if os.path.exists(jobOutputPath):
                            jobOutputDocumentObject = minidom.parse(jobOutputPath)
                            rootNode = jobOutputDocumentObject.documentElement
@@ -186,7 +215,7 @@ def processPegasusFiles(instanceDirectory,
       
                            jobOutputDocumentObject.unlink()
                         else:
-                           log("Job output file missing: %s" % (jobOutputPath))
+                           log("processPegasusFiles:Job output file missing: %s" % (jobOutputPath))
 
                         fpTimeResults = open(jobFiles[row[columnJob]],'a')
                         if fpTimeResults:
@@ -206,17 +235,79 @@ def processPegasusFiles(instanceDirectory,
                   log(traceback.format_exc())
                fpJobStats.close()
             else:
-               log("Failed to open Job Stats: %s" % (jobStatsPath))
+               log("processPegasusFiles:Failed to open Job Stats: %s" % (jobStatsPath))
          else:
-            log("Job Stats missing: %s" % (jobStatsPath))
+            log("processPegasusFiles:Job Stats missing: %s" % (jobStatsPath))
       else:
-         log("Failed to open Job State Log: %s" % (jobStateLogPath))
+         log("processPegasusFiles:Failed to open Job State Log: %s" % (jobStateLogPath))
    else:
-      log("Job State Log missing for post-processing: %s" % (jobStateLogPath))
+      log("processPegasusFiles:Job State Log missing for post-processing: %s" % (jobStateLogPath))
 
 
-def getInProgressPegasusJobStdFiles(instanceDirectory):
-   jobStateLogPath = os.path.join(instanceDirectory,'work','pegasus','jobstate.log')
+def getInProgressPegasusJobExitCodes(instanceDirectory,
+                                     scratchDirectory):
+   exitCodes = {}
+   pegasusFileDirectory = os.path.join(scratchDirectory,'work')
+   jobStateLogPath = os.path.join(pegasusFileDirectory,'jobstate.log')
+   if not os.path.exists(jobStateLogPath):
+      pegasusFileDirectory = os.path.join(instanceDirectory,'work')
+      jobStateLogPath = os.path.join(pegasusFileDirectory,'jobstate.log')
+   if os.path.exists(jobStateLogPath):
+      fpJobStateLog = open(jobStateLogPath,'rb')
+      if fpJobStateLog:
+         while True:
+            record = fpJobStateLog.readline()
+            if not record:
+               break
+            record = record.strip()
+
+            if   'SUBMIT' in record:
+               parts = record.split()
+               jobName,remoteSite = parts[1],parts[4]
+               if remoteSite != 'local' and remoteSite != '-':
+                  jobId = jobName.split('.')[0]
+                  instance = int(jobId.split('_')[1])
+                  if instance in exitCodes:
+                     del exitCodes[instance]
+            elif 'JOB_TERMINATED' in record:
+               parts = record.split()
+               jobName,remoteSite = parts[1],parts[4]
+               if remoteSite != 'local' and remoteSite != '-':
+                  jobId = jobName.split('.')[0]
+                  instance = int(jobId.split('_')[1])
+                  exitCodes[instance] = 'D'
+            elif 'JOB_SUCCESS' in record:
+               parts = record.split()
+               jobName,remoteSite = parts[1],parts[4]
+               if remoteSite != 'local' and remoteSite != '-':
+                  jobId = jobName.split('.')[0]
+                  instance = int(jobId.split('_')[1])
+                  if instance in exitCodes:
+                     exitCodes[instance] = 'D'
+            elif 'JOB_FAILURE' in record:
+               parts = record.split()
+               jobName,remoteSite = parts[1],parts[4]
+               if remoteSite != 'local' and remoteSite != '-':
+                  jobId = jobName.split('.')[0]
+                  instance = int(jobId.split('_')[1])
+                  if instance in exitCodes:
+                     exitCodes[instance] = 'EF'
+         fpJobStateLog.close()
+      else:
+         log("getInProgressPegasusJobExitCodes:Failed to open Job State Log: %s" % (jobStateLogPath))
+#  else:
+#     log("getInProgressPegasusJobExitCodes:Job State Log missing: %s" % (jobStateLogPath))
+
+   return(exitCodes)
+
+
+def getInProgressPegasusJobStdFiles(instanceDirectory,
+                                    scratchDirectory):
+   pegasusFileDirectory = os.path.join(scratchDirectory,'work')
+   jobStateLogPath = os.path.join(pegasusFileDirectory,'jobstate.log')
+   if not os.path.exists(jobStateLogPath):
+      pegasusFileDirectory = os.path.join(instanceDirectory,'work')
+      jobStateLogPath = os.path.join(pegasusFileDirectory,'jobstate.log')
    if os.path.exists(jobStateLogPath):
       fpJobStateLog = open(jobStateLogPath,'rb')
       if fpJobStateLog:
@@ -229,17 +320,18 @@ def getInProgressPegasusJobStdFiles(instanceDirectory):
                parts = record.split()
                jobName,remoteSite = parts[1],parts[4]
                jobId = jobName.split('.')[0]
+               runNameId = '_'.join(jobName.split('_')[-2:])
                jobTry = 0
                if remoteSite != 'local' and remoteSite != '-':
                   jobOutputFile = "%s.out.%s" % (jobName,str(jobTry).zfill(3))
-                  jobOutputPath = os.path.join(instanceDirectory,'work','pegasus',jobOutputFile)
+                  jobOutputPath = os.path.join(pegasusFileDirectory,jobOutputFile)
                   if os.path.exists(jobOutputPath):
                      jobOutputDocumentObject = minidom.parse(jobOutputPath)
                      rootNode = jobOutputDocumentObject.documentElement
    
                      jobStdout = getJobOutputContent(rootNode,'statcall','stdout','data')
                      if jobStdout:
-                        jobStdoutFile = os.path.join(instanceDirectory,'InProcessResults',"%s.stdout" % (jobId))
+                        jobStdoutFile = os.path.join(instanceDirectory,'InProcessResults',"%s.stdout" % (runNameId))
                         fpStdout = open(jobStdoutFile,'w')
                         fpStdout.write(jobStdout)
                         fpStdout.close()
@@ -247,24 +339,29 @@ def getInProgressPegasusJobStdFiles(instanceDirectory):
    
                      jobStderr = getJobOutputContent(rootNode,'statcall','stderr','data')
                      if jobStderr:
-                        jobStderrFile = os.path.join(instanceDirectory,'InProcessResults',"%s.stderr" % (jobId))
+                        jobStderrFile = os.path.join(instanceDirectory,'InProcessResults',"%s.stderr" % (runNameId))
                         fpStderr = open(jobStderrFile,'w')
                         fpStderr.write(jobStderr)
                         fpStderr.close()
                      del jobStderr
                      jobOutputDocumentObject.unlink()
                   else:
-                     log("Job output file missing: %s" % (jobOutputPath))
+                     log("getInProgressPegasusJobStdFiles:Job output file missing: %s" % (jobOutputPath))
          fpJobStateLog.close()
       else:
-         log("Failed to open Job State Log: %s" % (jobStateLogPath))
+         log("getInProgressPegasusJobStdFiles:Failed to open Job State Log: %s" % (jobStateLogPath))
    else:
-      log("Job State Log missing: %s" % (jobStateLogPath))
+      log("getInProgressPegasusJobStdFiles:Job State Log missing: %s" % (jobStateLogPath))
 
 
 def getPegasusStdTimeFiles(instanceDirectory,
+                           scratchDirectory,
                            dagTimeResultsFile):
-   jobStateLogPath = os.path.join(instanceDirectory,'work','pegasus','jobstate.log')
+   pegasusFileDirectory = os.path.join(scratchDirectory,'work')
+   jobStateLogPath = os.path.join(pegasusFileDirectory,'jobstate.log')
+   if not os.path.exists(jobStateLogPath):
+      pegasusFileDirectory = os.path.join(instanceDirectory,'work')
+      jobStateLogPath = os.path.join(pegasusFileDirectory,'jobstate.log')
    if os.path.exists(jobStateLogPath):
       condorIds = {}
       fpJobStateLog = open(jobStateLogPath,'rb')
@@ -318,6 +415,7 @@ def getPegasusStdTimeFiles(instanceDirectory,
                   for row in csvReader:
                      if row[columnSite] != 'local' and row[columnSite] != '-':
                         jobName = row[columnJob]
+                        runNameId = '_'.join(jobName.split('_')[-2:])
                         jobId = jobName.split('.')[0]
                         jobFileIndex = jobId.split('_')[1]
                         if not jobId in jobFiles:
@@ -339,14 +437,14 @@ def getPegasusStdTimeFiles(instanceDirectory,
 #                                                        row[columnRuntime]))
 
                         jobOutputFile = "%s.out.%s" % (row[columnJob],str(jobTry).zfill(3))
-                        jobOutputPath = os.path.join(instanceDirectory,'work','pegasus',jobOutputFile)
+                        jobOutputPath = os.path.join(pegasusFileDirectory,jobOutputFile)
                         if os.path.exists(jobOutputPath):
                            jobOutputDocumentObject = minidom.parse(jobOutputPath)
                            rootNode = jobOutputDocumentObject.documentElement
 
                            jobStdout = getJobOutputContent(rootNode,'statcall','stdout','data')
                            if jobStdout:
-                              jobStdoutFile = os.path.join(instanceDirectory,jobFileIndex,"%s.stdout" % (jobId))
+                              jobStdoutFile = os.path.join(instanceDirectory,jobFileIndex,"%s.stdout" % (runNameId))
                               fpStdout = open(jobStdoutFile,'w')
                               fpStdout.write(jobStdout)
                               fpStdout.close()
@@ -354,7 +452,7 @@ def getPegasusStdTimeFiles(instanceDirectory,
 
                            jobStderr = getJobOutputContent(rootNode,'statcall','stderr','data')
                            if jobStderr:
-                              jobStderrFile = os.path.join(instanceDirectory,jobFileIndex,"%s.stderr" % (jobId))
+                              jobStderrFile = os.path.join(instanceDirectory,jobFileIndex,"%s.stderr" % (runNameId))
                               fpStderr = open(jobStderrFile,'w')
                               fpStderr.write(jobStderr)
                               fpStderr.close()
@@ -370,7 +468,7 @@ def getPegasusStdTimeFiles(instanceDirectory,
       
                            jobOutputDocumentObject.unlink()
                         else:
-                           log("Job output file missing: %s" % (jobOutputPath))
+                           log("getPegasusStdTimeFiles:Job output file missing: %s" % (jobOutputPath))
 
                         fpTimeResults = open(jobFiles[jobId],'a')
                         if fpTimeResults:
@@ -390,12 +488,12 @@ def getPegasusStdTimeFiles(instanceDirectory,
                   log(traceback.format_exc())
                fpJobStats.close()
             else:
-               log("Failed to open Job Stats: %s" % (jobStatsPath))
+               log("getPegasusStdTimeFiles:Failed to open Job Stats: %s" % (jobStatsPath))
          else:
-            log("Job Stats missing: %s" % (jobStatsPath))
+            log("getPegasusStdTimeFiles:Job Stats missing: %s" % (jobStatsPath))
       else:
-         log("Failed to open Job State Log: %s" % (jobStateLogPath))
+         log("getPegasusStdTimeFiles:Failed to open Job State Log: %s" % (jobStateLogPath))
    else:
-      log("Job State Log missing for post-processing: %s" % (jobStateLogPath))
+      log("getPegasusStdTimeFiles:Job State Log missing for post-processing: %s" % (jobStateLogPath))
 
 
